@@ -17,25 +17,29 @@ import os
 
 class TestReader(BaseReader):
 
-    def __init__(self,*sheets):
-        self.book = DummyBook()
+    formatting_info = 0
+    
+    def __init__(self,*sheets,**books):
+        self.setup(*sheets,**books)
+        
+    def setup(self,*sheets,**books):
+        self.books = []
+        if sheets:
+            self.makeBook('test',sheets)
+        for name,value in sorted(books.items()):
+            self.makeBook(name,value)
+        
+    def makeBook(self,sheet_name,sheets):
+        book = DummyBook(self.formatting_info)
         index = 0
         for name,rows in sheets:
-            make_sheet(rows,self.book,name,index)
+            make_sheet(rows,book,name,index)
             index+=1
-            
-    def get_workbooks(self):
-        yield self.book,'test.xls'
-
-class TestTestReader(TestCase):
-
-    def test_cell_type(self):
-        r = TestReader(('Sheet1',(((XL_CELL_NUMBER,0.0),),)))
-        book = tuple(r.get_workbooks())[0][0]
-        cell = book.sheet_by_index(0).cell(0,0)
-        self.assertEqual(cell.value,0.0)
-        self.assertEqual(cell.ctype,XL_CELL_NUMBER)
+        self.books.append((book,sheet_name+'.xls'))
         
+    def get_workbooks(self):
+        return self.books
+
 class TestBaseReader(TestCase):
 
     def test_no_implementation(self):
@@ -267,8 +271,10 @@ class TestMethodFilter(TestCase):
             ])
 
     def test_invalid(self):
-        m = should_raise(OurMethodFilter,ValueError("'foo' is not a valid method name"))        
-        m(self.called,['foo'])
+        s = should_raise(OurMethodFilter)
+        f = s(self.called,['foo'])
+        compare(s.raised,C(ValueError("'foo' is not a valid method name")))
+        
     
 from xlutils.filter import Echo
 
@@ -1036,6 +1042,118 @@ class TestExamples(TestCase):
         font = actual.font_list[actual.xf_list[cell.xf_index].font_index]
         self.assertEqual(font.underline_type,1)
             
+class TestTestReader(TestCase):
+
+    def test_cell_type(self):
+        r = TestReader(('Sheet1',(((XL_CELL_NUMBER,0.0),),)))
+        book = tuple(r.get_workbooks())[0][0]
+        cell = book.sheet_by_index(0).cell(0,0)
+        self.assertEqual(cell.value,0.0)
+        self.assertEqual(cell.ctype,XL_CELL_NUMBER)
+        
+    def test(self):
+        r = TestReader(
+            test1=[('Sheet1',[['R1C1','R1C2'],
+                              ['R2C1','R2C2']]),
+                   ('Sheet2',[['R3C1','R3C2'],
+                              ['R4C1','R4C2']])],
+            test2=[('Sheet3',[['R5C1','R5C2'],
+                              ['R6C1','R6C2']]),
+                   ('Sheet4',[['R7C1','R7C2'],
+                              ['R8C1','R8C2']])],
+            )
+        f = Mock()
+        r(f)
+        compare([
+            ('start', (), {}),
+            ('workbook', (C('xlutils.tests.fixtures.DummyBook'), 'test1.xls'), {}),
+            ('sheet', (C('xlrd.sheet.Sheet'), 'Sheet1'), {}),
+            ('row', (0, 0), {}),
+            ('cell', (0, 0, 0, 0), {}),
+            ('cell',(0, 1, 0, 1), {}),
+            ('row', (1, 1), {}),
+            ('cell', (1, 0, 1, 0), {}),
+            ('cell',(1, 1, 1, 1), {}),
+            ('sheet', (C('xlrd.sheet.Sheet'), 'Sheet2'), {}),
+            ('row', (0, 0), {}),
+            ('cell', (0, 0, 0, 0), {}),
+            ('cell',(0, 1, 0, 1), {}),
+            ('row', (1, 1), {}),
+            ('cell', (1, 0, 1, 0), {}),
+            ('cell',(1, 1, 1, 1), {}),
+            ('workbook', (C('xlutils.tests.fixtures.DummyBook'), 'test2.xls'), {}),
+            ('sheet', (C('xlrd.sheet.Sheet'), 'Sheet3'), {}),
+            ('row', (0, 0), {}),
+            ('cell', (0, 0, 0, 0), {}),
+            ('cell',(0, 1, 0, 1), {}),
+            ('row', (1, 1), {}),
+            ('cell', (1, 0, 1, 0), {}),
+            ('cell',(1, 1, 1, 1), {}),
+            ('sheet', (C('xlrd.sheet.Sheet'), 'Sheet4'), {}),
+            ('row', (0, 0), {}),
+            ('cell', (0, 0, 0, 0), {}),
+            ('cell',(0, 1, 0, 1), {}),
+            ('row', (1, 1), {}),
+            ('cell', (1, 0, 1, 0), {}),
+            ('cell',(1, 1, 1, 1), {}),
+            ('finish', (), {})
+            ],f.method_calls)
+        
+    def test_setup(self):
+        r = TestReader()
+        f = Mock()
+        r(f)
+        compare([('start', (), {}), ('finish', (), {})],f.method_calls)
+        r.setup(('Sheet1',[['R1C1']]),
+                test1=[('Sheet2',[['R2C1']])])
+        r(f)
+        compare([
+            ('start', (), {}),
+            ('finish', (), {}),
+            ('start', (), {}),
+            ('workbook',(C('xlutils.tests.fixtures.DummyBook'), 'test.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet'), 'Sheet1'), {}),
+            ('row', (0, 0), {}),
+            ('cell', (0, 0, 0, 0), {}),
+            ('workbook',(C('xlutils.tests.fixtures.DummyBook'), 'test1.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet'), 'Sheet2'), {}),
+            ('row', (0, 0), {}),
+            ('cell', (0, 0, 0, 0), {}),
+            ('finish', (), {})],f.method_calls)
+        
+    def test_formatting_info(self):
+        r = TestReader()
+        f = Mock()
+        
+        r.formatting_info = True
+        
+        r.setup(('Sheet1',[['R1C1','R1C2']]))
+
+        # at this point you can now manipulate the xf index as follows:
+        book = r.books[0][0]
+        sx,rx,cx = 0,0,0
+        book.sheet_by_index(sx)._cell_xf_indexes[rx][cx]=42
+
+        # NB: cells where you haven't specified an xf index manually as
+        #     above will have an xf index of 0:
+        compare(book.sheet_by_index(0).cell(0,1).xf_index,0)
+        # and no matching style:
+        should_raise(book.xf_list,IndexError)[0]
+
+        r(f)
+        
+        compare([
+            ('start', (), {}),
+            ('workbook',(C('xlutils.tests.fixtures.DummyBook'), 'test.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet'), 'Sheet1'), {}),
+            ('row', (0, 0), {}),
+            ('cell', (0, 0, 0, 0), {}),
+            ('cell', (0, 1, 0, 1), {}),
+            ('finish', (), {})],f.method_calls)
+
+        compare(book.sheet_by_index(0).cell(0,0).xf_index,42)
+        
+        
 def test_suite():
     return TestSuite((
         makeSuite(TestBaseReader),
